@@ -4,10 +4,7 @@ rbind.tbl_ts <- function(...) {
   data <- list2(...)[[1]]
   x <- dplyr::bind_rows(...)
   x <- retain_tsibble(x, key(data), index(data))
-  build_tsibble_meta(
-    x, key = key(data), index = !! index(data),
-    index2 = !! index2(data), regular = is_regular(data)
-  )
+  update_meta(x, data, ordered = NULL)
 }
 
 #' @method cbind tbl_ts
