@@ -4,18 +4,6 @@ format.tbl_ts <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
   format(tibble::trunc_mat(x, n = n, width = width, n_extra = n_extra))
 }
 
-#' @export
-glimpse.tbl_ts <- function(x, width = NULL, ...) {
-  idx <- index(x)
-  t_span <- paste(range(dplyr::pull(x, !! idx)), collapse = " ~ ")
-  cat_line(t_span)
-  build_tsibble_meta(
-    NextMethod(), key = key(x), index = !! idx, index2 = !! index2(x), 
-    ordered = is_ordered(x), regular = is_regular(x)
-  )
-  invisible(x)
-}
-
 #' @export
 print.interval <- function(x, digits = NULL, ...) {
   cat_line(format(x, digits = digits, ...))
