@@ -1,3 +1,15 @@
+abort_stretch_size <- function(...) {
+  dots <- dots_list(...)
+  if (".size" %in% names(dots)) {
+    abort("Argument `.size` is retired. Please use `.step`.")
+  }
+}
+
+nrow2 <- function(.x) {
+  if (is.data.frame(.x)) .x <- as.list(.x)
+  NROW(.x)
+}
+
 #' Stretching window calculation
 #'
 #' Fixing an initial window and expanding more observations:
@@ -8,7 +20,9 @@
 #'
 #' @inheritParams slide
 #' @param .init A positive integer for an initial window size.
+#' @param .step A positive integer for incremental step.
 #'
+#' @return if `.fill != NULL`, it always returns the same length as input.
 #' @rdname stretch
 #' @export
 #' @family stretching window functions
@@ -19,20 +33,24 @@
 #'
 #' @examples
 #' x <- 1:5
+#' stretch_dbl(x, mean, .step = 2)
+#' stretch_lgl(x, ~ mean(.) > 2, .step = 2)
 #' lst <- list(x = x, y = 6:10, z = 11:15)
-#' stretch_dbl(x, mean, .size = 2)
-#' stretch_lgl(x, ~ mean(.) > 2, .size = 2)
-#' stretch(lst, ~ ., .size = 2)
-stretch <- function(.x, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
-  lst_x <- stretcher(.x, .size = .size, .init = .init, .bind)
-  map(lst_x, .f, ...)
+#' stretch(lst, ~ ., .step = 2, .fill = NULL)
+stretch <- function(.x, .f, ..., .step = 1, .init = 1, .fill = NA,
+  .bind = FALSE) {
+  abort_stretch_size(...)
+  lst_x <- stretcher(.x, .step = .step, .init = .init, .bind = .bind)
+  out <- map(lst_x, .f, ...)
+  pad_stretch(out, .init = .init, .step = .step, .fill = .fill,
+    expect_length = nrow2(.x))
 }
 
 #' @evalRd paste0('\\alias{stretch_', c("lgl", "chr", "dbl", "int"), '}')
 #' @name stretch
 #' @rdname stretch
 #' @exportPattern ^stretch_
-for(type in c("lgl", "chr", "dbl", "int")){
+for (type in c("lgl", "chr", "dbl", "int")) {
   assign(
     paste0("stretch_", type),
     replace_fn_names(stretch, list(map = paste0("map_", type)))
@@ -42,23 +60,24 @@ for(type in c("lgl", "chr", "dbl", "int")){
 #' @rdname stretch
 #' @export
 stretch_dfr <- function(
-  .x, .f, ..., .size = 1, .init = 1, .bind = FALSE, .id = NULL
+  .x, .f, ..., .step = 1, .init = 1, .fill = NA, .bind = FALSE, .id = NULL
 ) {
   out <- stretch(
-    .x, .f = .f, ..., .size = .size, .init = .init, 
+    .x, .f = .f, ..., .step = .step, .init = .init, .fill = .fill,
     .bind = .bind
   )
-  dplyr::bind_rows(!!! out, .id = .id)
+  bind_df(out, .size = .init, .fill = .fill, .id = .id)
 }
 
 #' @rdname stretch
 #' @export
-stretch_dfc <- function(.x, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
+stretch_dfc <- function(.x, .f, ..., .step = 1, .init = 1, .fill = NA,
+  .bind = FALSE) {
   out <- stretch(
-    .x, .f = .f, ..., .size = .size, .init = .init,
+    .x, .f = .f, ..., .step = .step, .init = .init, .fill = .fill,
     .bind = .bind
   )
-  dplyr::bind_cols(!!! out)
+  bind_df(out, .size = .init, .fill = .fill, byrow = FALSE)
 }
 
 #' Stretching window calculation over multiple simultaneously
@@ -70,7 +89,7 @@ stretch_dfc <- function(.x, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
 #' * `stretch2_dfr()` `stretch2_dfc()` return data frames using row-binding & column-binding.
 #'
 #' @inheritParams slide2
-#' @param .size,.init An integer for moving and initial window size.
+#' @inheritParams stretch
 #'
 #' @rdname stretch2
 #' @export
@@ -85,11 +104,11 @@ stretch_dfc <- function(.x, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
 #' z <- 11:15
 #' lst <- list(x = x, y = y, z = z)
 #' df <- as.data.frame(lst)
-#' stretch2(x, y, sum, .size = 2)
-#' stretch2(lst, lst, ~ ., .size = 2)
-#' stretch2(df, df, ~ ., .size = 2)
-#' pstretch(lst, sum, .size = 1)
-#' pstretch(list(lst, lst), ~ ., .size = 2)
+#' stretch2(x, y, sum, .step = 2)
+#' stretch2(lst, lst, ~ ., .step = 2)
+#' stretch2(df, df, ~ ., .step = 2)
+#' pstretch(lst, sum, .step = 1)
+#' pstretch(list(lst, lst), ~ ., .step = 2)
 #'
 #' ###
 #' # row-wise stretching over data frame
@@ -97,20 +116,24 @@ stretch_dfc <- function(.x, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
 #'
 #' x <- as.Date("2017-01-01") + 0:364
 #' df <- data.frame(x = x, y = seq_along(x))
-#' 
+#'
 #' tibble(
 #'   data = pstretch(df, function(...) as_tibble(list(...)), .init = 10)
 #' )
-stretch2 <- function(.x, .y, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
-  lst <- pstretcher(.x, .y, .size = .size, .init = .init, .bind = .bind)
-  map2(lst[[1]], lst[[2]], .f, ...)
+stretch2 <- function(.x, .y, .f, ..., .step = 1, .init = 1, .fill = NA,
+  .bind = FALSE) {
+  abort_stretch_size(...)
+  lst <- pstretcher(.x, .y, .step = .step, .init = .init, .bind = .bind)
+  out <- map2(lst[[1]], lst[[2]], .f, ...)
+  pad_stretch(out, .init = .init, .step = .step, .fill = .fill,
+    expect_length = nrow2(recycle(list(.x, .y))[[1]]))
 }
 
 #' @evalRd paste0('\\alias{stretch2_', c("lgl", "chr", "dbl", "int"), '}')
 #' @name stretch2
 #' @rdname stretch2
 #' @exportPattern ^stretch2_
-for(type in c("lgl", "chr", "dbl", "int")){
+for (type in c("lgl", "chr", "dbl", "int")) {
   assign(
     paste0("stretch2_", type),
     replace_fn_names(stretch2, list(map2 = paste0("map2_", type)))
@@ -120,39 +143,44 @@ for(type in c("lgl", "chr", "dbl", "int")){
 #' @rdname stretch2
 #' @export
 stretch2_dfr <- function(
-  .x, .y, .f, ..., .size = 1, .init = 1, .bind = FALSE, .id = NULL
+  .x, .y, .f, ..., .step = 1, .init = 1, .fill = NA, .bind = FALSE, .id = NULL
 ) {
   out <- stretch2(
-    .x, .y, .f = .f, ..., .size = .size, .init = .init,
+    .x, .y, .f = .f, ..., .step = .step, .init = .init, .fill = .fill,
     .bind = .bind
   )
-  dplyr::bind_rows(!!! out, .id = .id)
+  bind_df(out, .size = .init, .fill = .fill, .id = .id)
 }
 
 #' @rdname stretch2
 #' @export
 stretch2_dfc <- function(
-  .x, .y, .f, ..., .size = 1, .init = 1, .bind = FALSE
+  .x, .y, .f, ..., .step = 1, .init = 1, .fill = NA, .bind = FALSE
 ) {
   out <- stretch2(
-    .x, .y, .f = .f, ..., .size = .size, .init = .init,
+    .x, .y, .f = .f, ..., .step = .step, .init = .init, .fill = .fill,
     .bind = .bind
   )
-  dplyr::bind_cols(!!! out)
+  bind_df(out, .size = .init, .fill = .fill, byrow = FALSE)
 }
 
 #' @rdname stretch2
 #' @export
-pstretch <- function(.l, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
-  lst <- pstretcher(!!! .l, .size = .size, .init = .init, .bind = .bind)
-  pmap(lst, .f, ...)
+pstretch <- function(.l, .f, ..., .step = 1, .init = 1, .fill = NA,
+  .bind = FALSE) {
+  abort_stretch_size(...)
+  lst <- pstretcher(!!! .l, .step = .step, .init = .init,
+    .bind = .bind)
+  out <- pmap(lst, .f, ...)
+  pad_stretch(out, .init = .init, .step = .step, .fill = .fill,
+    expect_length = nrow2(recycle(.l)[[1]]))
 }
 
 #' @evalRd paste0('\\alias{pstretch_', c("lgl", "chr", "dbl", "int"), '}')
 #' @name pstretch
 #' @rdname stretch2
 #' @exportPattern ^pstretch_
-for(type in c("lgl", "chr", "dbl", "int")){
+for (type in c("lgl", "chr", "dbl", "int")) {
   assign(
     paste0("pstretch_", type),
     replace_fn_names(pstretch, list(pmap = paste0("pmap_", type)))
@@ -162,17 +190,20 @@ for(type in c("lgl", "chr", "dbl", "int")){
 #' @rdname stretch2
 #' @export
 pstretch_dfr <- function(
-  .l, .f, ..., .size = 1, .init = 1, .bind = FALSE, .id = NULL
+  .l, .f, ..., .step = 1, .init = 1, .fill = NA, .bind = FALSE, .id = NULL
 ) {
-  out <- pstretch(.l, .f, ..., .size = .size, .init = .init, .bind = .bind)
-  dplyr::bind_rows(!!! out, .id = .id)
+  out <- pstretch(.l, .f, ..., .step = .step, .init = .init, .fill = .fill,
+    .bind = .bind)
+  bind_df(out, .size = .init, .fill = .fill, .id = .id)
 }
 
 #' @rdname stretch2
 #' @export
-pstretch_dfc <- function(.l, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
-  out <- pstretch(.l, .f, ..., .size = .size, .init = .init, .bind = .bind)
-  dplyr::bind_cols(!!! out)
+pstretch_dfc <- function(.l, .f, ..., .step = 1, .init = 1, .fill = NA,
+  .bind = FALSE) {
+  out <- pstretch(.l, .f, ..., .step = .step, .init = .init, .fill = .fill,
+    .bind = .bind)
+  bind_df(out, .size = .init, .fill = .fill, byrow = FALSE)
 }
 
 #' Split the input to a list according to the stretching window size.
@@ -189,27 +220,26 @@ pstretch_dfc <- function(.l, .f, ..., .size = 1, .init = 1, .bind = FALSE) {
 #' lst <- list(x = x, y = y, z = z)
 #' df <- as.data.frame(lst)
 #'
-#' stretcher(x, .size = 2)
-#' stretcher(lst, .size = 2)
-#' stretcher(df, .size = 2)
-#' pstretcher(df, df, .size = 2)
-stretcher <- function(.x, .size = 1, .init = 1, .bind = FALSE) {
-  bad_window_function(.size)
+#' stretcher(x, .step = 2)
+#' stretcher(lst, .step = 2)
+#' stretcher(df, .step = 2)
+#' pstretcher(df, df, .step = 2)
+stretcher <- function(.x, .step = 1, .init = 1, .bind = FALSE) {
+  bad_window_function(.step)
   if (!is_integerish(.init, n = 1) || .init < 1) {
     abort("`.init` must be a positive integer.")
   }
   abort_not_lst(.x, .bind = .bind)
   if (is.data.frame(.x)) .x <- as.list(.x)
   len_x <- NROW(.x)
-  abs_size <- abs(.size)
-  counter <- incr(init = .init, size = abs_size)
-  if (sign(.size) < 0) .x <- rev(.x)
-  ncall <- seq_len(ceiling((len_x - .init) / abs_size) - 1)
-  incr_lst <- c(
-    list(seq_len(.init)),
-    map(ncall, ~ seq_len(counter())),
-    list(seq_len(len_x))
-  )
+  if (len_x <= .init) {
+    abort(sprintf("`.init` must be less than %s.", len_x))
+  }
+  abs_size <- abs(.step)
+  counter <- incr(.init = .init, .step = abs_size)
+  if (sign(.step) < 0) .x <- rev(.x)
+  ncall <- seq_len(floor((len_x - .init) / abs_size))
+  incr_lst <- c(list(seq_len(.init)), map(ncall, ~ seq_len(counter())))
   out <- map(incr_lst, function(idx) .x[idx])
   if (.bind) bind_lst(out) else out
 }
@@ -217,29 +247,76 @@ stretcher <- function(.x, .size = 1, .init = 1, .bind = FALSE) {
 
 #' @rdname stretcher
 #' @export
-pstretcher <- function(..., .size = 1, .init = 1, .bind = FALSE) { # parallel sliding
+pstretcher <- function(..., .step = 1, .init = 1, .bind = FALSE) { # parallel sliding
+  abort_stretch_size(...)
   lst <- recycle(list2(...))
-  map(lst, function(x) stretcher(x, .size, .init, .bind))
+  map(lst, function(x) stretcher(x, .step, .init, .bind))
 }
 
-incr <- function(init, size) {
-  init
+#' Perform stretching windows on a tsibble by row
+#'
+#' @param .x A tsibble.
+#' @param .step A positive integer for incremental step.
+#' @inheritParams stretch
+#' @param .id A character naming the new column `.id` containing the partition.
+#'
+#' @inheritSection slide_tsibble Rolling tsibble
+#' @family rolling tsibble
+#' @export
+#' @examples
+#' harvest <- tsibble(
+#'   year = rep(2010:2012, 2),
+#'   fruit = rep(c("kiwi", "cherry"), each = 3),
+#'   kilo = sample(1:10, size = 6),
+#'   key = id(fruit), index = year
+#' )
+#' harvest %>%
+#'   stretch_tsibble()
+stretch_tsibble <- function(.x, .step = 1, .init = 1, .id = ".id") {
+  lst_indices <- map(key_rows(.x), stretcher, .step = .step, .init = .init)
+  roll_tsibble(.x, indices = lst_indices, .id = .id)
+}
+
+incr <- function(.init, .step) {
+  .init
   function() {
-    init <<- init + size
-    init
+    .init <<- .init + .step
+    .init
+  }
+}
+
+pad_stretch <- function(x, .init = 1, .step = 1, .fill = NA,
+  expect_length = NULL) {
+  if (is_null(.fill)) return(x)
+
+  len_x <- length(x)
+  fill_size <- abs(.init - .step)
+  if (.step == 1) {
+    c(rep(.fill, fill_size), x)
+  } else {
+    seq_x <- seq_len(len_x)
+    rep_idx <- rep.int(len_x + 1, len_x * (.step - 1))
+    null_idx <- matrix(rep_idx, nrow = .step - 1)
+    idx <- as.integer(rbind(seq_x, null_idx, deparse.level = 0))
+    res <- x[idx]
+    res[!(idx %in% seq_x)] <- .fill
+    if (.init > 1) {
+      res <- c(rep(.fill, .init - 1), res)
+    }
+    res[seq_len(expect_length)]
   }
 }
 
 #' Stretching window in parallel
 #'
 #' Multiprocessing equivalents of [slide()], [tile()], [stretch()] prefixed by `future_`.
-#' * Variants for corresponding types: `future_*_lgl()`, `future_*_int()`, 
+#' * Variants for corresponding types: `future_*_lgl()`, `future_*_int()`,
 #' `future_*_dbl()`, `future_*_chr()`, `future_*_dfr()`, `future_*_dfc()`.
-#' * Extra arguments `.progress` and `.options` for enabling progress bar and the 
-#' future specific options to use with the workers. 
+#' * Extra arguments `.progress` and `.options` for enabling progress bar and the
+#' future specific options to use with the workers.
 #'
 #' @evalRd {suffix <- c("lgl", "chr", "int", "dbl", "dfr", "dfc"); c(paste0('\\alias{future_', c("stretch", "stretch2", "pstretch"), '}'), paste0('\\alias{future_stretch_', suffix, '}'), paste0('\\alias{future_stretch2_', suffix, '}'), paste0('\\alias{future_pstretch_', suffix, '}'))}
-#' @name future_stretch
+#' @name future_stretch()
 #' @rdname future-stretch
 #' @exportPattern ^future_
 # nocov start
