@@ -18,71 +18,28 @@
 
 #' @export
 `[.tbl_ts` <- function(x, i, j, drop = FALSE) {
-  n_args <- nargs() - !missing(drop)
+  res <- NextMethod()
+  if (!is.data.frame(res)) return(res)
 
-  if (n_args <= 2) { # only column subsetting
-    if (!missing(drop)) {
-      warn("`drop` is ignored.")
-    }
+  cn <- names(res)
+  new_key <- syms(cn[cn %in% key_vars(x)])
 
-    ordered <- is_ordered(x)
-    if (!missing(i)) { # x[1:2]
-      i <- tidyselect::vars_select(names(x), i)
-      lgl_i <- has_index(i, x) && (n_keys(x) < 2 || has_any_key(i, x))
-      result <- .subset(x, i)
-      x <- remove_key(x, i)
-      if (is_false(lgl_i)) {
-        return(as_tibble(NextMethod()))
-      } else {
-        return(build_tsibble_meta(
-          result, key = key(x), index = !! index(x), index2 = !! index2(x),
-          regular = is_regular(x), ordered = ordered
-        ))
-      }
-    } else { # e.g. x[]
-      result <- x
-      return(build_tsibble_meta(
-        result, key = key(x), index = !! index(x), index2 = !! index2(x),
-        regular = is_regular(x), ordered = ordered, interval = interval(x)
-      ))
-    }
-  }
-
-  # subset by columns
-  if (!missing(j)) {
-    chr_j <- tidyselect::vars_select(names(x), j)
-    lgl_j <- has_index(chr_j, x) && (n_keys(x) < 2 || has_any_key(chr_j, x))
-    result <- .subset(x, j)
-    if (is_false(lgl_j)) {
-      return(as_tibble(NextMethod()))
-    }
-    x <- remove_key(x, chr_j)
-  } else {
-    result <- x
-  }
-
-  ordered <- is_ordered(x)
   if (!missing(i)) {
-    # ordered <- row_validate(i)
-    result <- purrr::map(result, `[`, i)
-    if (any(i > NROW(x))) {
-      return(as_tibble(NextMethod()))
-    }
-    nr <- length(result[[1]])
+    if (any_not_equal_to_c(anyDuplicated.default(i), 0)) return(as_tibble(res))
   }
 
-  if (drop) {
-    if (has_length(result, 1)) {
-      return(result[[1L]])
-    } else if (nr == 1L) {
-      return(result)
-    }
+  not_tsibble <- !(index_var(x) %in% cn) || NROW(res) > NROW(x)
+  maybe_tsibble <- n_keys(x) > 1 && !all(is.element(key(x), new_key))
+  if (not_tsibble || maybe_tsibble) {
+    as_tibble(res)
+  } else {
+    reg <- is_regular(x)
+    ord <- is_ordered(x)
+    build_tsibble(
+      res, key = new_key, index = !! index(x), index2 = !! index2(x),
+      regular = reg, ordered = ord, validate = FALSE
+    )
   }
-
-  build_tsibble_meta(
-    result, key = key(x), index = !! index(x), index2 = !! index2(x),
-    regular = is_regular(x), ordered = ordered, interval = NULL
-  )
 }
 
 # #' @export
