@@ -14,8 +14,8 @@ as_tsibble.ts <- function(x, tz = "UTC", ...) {
   idx <- time_to_date(x, tz = tz)
   value <- as.numeric(x) # rm its ts class
   tbl <- tibble(index = idx, value = value)
-  build_tsibble_meta(
-    tbl, key = id(), index = index, ordered = TRUE
+  build_tsibble(
+    tbl, key = id(), index = index, ordered = TRUE, validate = FALSE
   )
 }
 
@@ -32,13 +32,13 @@ as_tsibble.ts <- function(x, tz = "UTC", ...) {
 as_tsibble.mts <- function(x, tz = "UTC", gather = TRUE, ...) {
   if (gather) {
     long_tbl <- gather_ts(x, tz = tz)
-    build_tsibble_meta(
-      long_tbl, key = id(key), index = index, ordered = TRUE
+    build_tsibble(
+      long_tbl, key = id(key), index = index, ordered = TRUE, validate = FALSE
     )
   } else {
     wide_tbl <- bind_time(x, tz = tz)
-    build_tsibble_meta(
-      wide_tbl, key = id(), index = index, ordered = TRUE
+    build_tsibble(
+      wide_tbl, key = id(), index = index, ordered = TRUE, validate = FALSE
     )
   }
 }
@@ -63,7 +63,8 @@ as_tsibble.hts <- function(x, tz = "UTC", ...) {
   tbl_hts <- dplyr::bind_cols(tbl, full_labs)
   # this would work around the special character issue in headers for parse()
   key <- syms(colnames(tbl_hts)[3:ncol(tbl_hts)])
-  build_tsibble_meta(tbl_hts, key = key, index = index, ordered = TRUE)
+  build_tsibble(tbl_hts, key = key, index = index, ordered = TRUE,
+    validate = FALSE)
 }
 
 # as_tsibble.gts <- function(x, tz = "UTC", ...) {
