@@ -41,7 +41,8 @@ as.ts.tbl_ts <- function(x, value, frequency = NULL, fill = NA, ...) {
   idx <- index(x)
   tsbl_sel <- x %>% 
     arrange(!!! key_vars, !! idx) %>% 
-    select_tsibble(!! idx, !!! key_vars, !! value_var)
+    select_tsibble(!! idx, !!! key_vars, !! value_var, validate = FALSE) %>% 
+    as_tibble()
   if (is_empty(key_vars)) {
     finalise_ts(tsbl_sel, index = index(x), frequency = frequency)
   } else {
@@ -143,7 +144,7 @@ guess_frequency.numeric <- function(x) {
 
 #' @export
 guess_frequency.yearweek <- function(x) {
-  52 / pull_interval(x)$month
+  52 / pull_interval(x)$week
 }
 
 #' @export
