@@ -1,3 +1,7 @@
+#' @importFrom tibble type_sum
+#' @export
+tibble::type_sum
+
 #' @export
 type_sum.tbl_ts <- function(x) {
   "tsibble"
@@ -18,6 +22,10 @@ type_sum.yearquarter <- function(x) {
   "qtr"
 }
 
+#' @importFrom tibble tbl_sum
+#' @export
+tibble::tbl_sum
+
 #' @export
 tbl_sum.tbl_ts <- function(x) {
   int_x <- interval(x)
@@ -40,7 +48,7 @@ tbl_sum.tbl_ts <- function(x) {
 
 #' @export
 tbl_sum.grouped_ts <- function(x) {
-  n_grps <- big_mark(n_groups(x))
+  n_grps <- big_mark(length(group_rows(x)))
   if (n_grps == 0) {
     n_grps <- "?"
   }
@@ -48,7 +56,8 @@ tbl_sum.grouped_ts <- function(x) {
   idx2 <- quo_name(index2(x))
   grp_var <- setdiff(grps, idx2)
   idx_suffix <- paste("@", idx2)
-  res <- NextMethod()
+  res_grps <- NextMethod()
+  res <- res_grps[head(names(res_grps), -1L)] # rm "Groups"
   n_grps <- surround(n_grps, "[")
   if (is_empty(grp_var)) {
     c(res, "Groups" = paste(idx_suffix, n_grps))
