@@ -2,7 +2,9 @@
 # ref: tibble:::big_mark
 big_mark <- function(x, ...) {
   mark <- if (identical(getOption("OutDec"), ",")) "." else ","
-  formatC(x, big.mark = mark, ...)
+  ret <- formatC(x, big.mark = mark, ...)
+  ret[is.na(x)] <- "??"
+  ret
 }
 
 # ref: tibble:::cat_line
@@ -35,7 +37,7 @@ surround <- function(x, bracket = "(") {
 # inlined from https://github.com/r-lib/cli/blob/master/R/utf8.R
 is_utf8_output <- function() {
   opt <- getOption("cli.unicode", NULL)
-  if (! is.null(opt)) {
+  if (! is_null(opt)) {
     isTRUE(opt)
   } else {
     l10n_info()$`UTF-8` && !is_latex_output()
