@@ -15,11 +15,19 @@ first_arg <- function(x) {
 gcd_interval <- function(x) {
   if (has_length(x, 1)) { # only one time index
     NA_real_
-  } else {
+  } else if (is_integerish(x)) {
     gcd_vector(x)
+  } else {
+    gcd_vector_r(unique(abs(diff(x))))
   }
 }
 
+gcd2 <- function(a, b) {
+  if (isTRUE(all.equal(b, 0))) a else gcd2(b, a %% b)
+}
+
+gcd_vector_r <- function(x) Reduce(gcd2, x)
+
 min0 <- function(...) {
   min(..., na.rm = TRUE)
 }
