@@ -20,7 +20,7 @@ register_s3_method <- function(pkg, generic, class, fun = NULL) {
   stopifnot(is.character(generic), length(generic) == 1)
   stopifnot(is.character(class), length(class) == 1)
 
-  if (is.null(fun)) {
+  if (is_null(fun)) {
     fun <- get(paste0(generic, ".", class), envir = parent.frame())
   } else {
     stopifnot(is.function(fun))
