@@ -13,7 +13,8 @@ test_that("a tbl_df/data.frame", {
 test_that("unknown interval", {
   tsbl <- as_tsibble(dat_x[1, ], index = date)
   expect_identical(fill_gaps(tsbl), tsbl)
-  expect_equal(count_gaps(tsbl)$.n, 0)
+  expect_equal(count_gaps(tsbl)$.n, integer(0))
+  expect_identical(scan_gaps(pedestrian[0L, ]), pedestrian[0L, 1:2])
   expect_equal(has_gaps(tsbl), tibble(".gaps" = FALSE))
 })
 
@@ -27,7 +28,10 @@ test_that("an irregular tbl_ts", {
 test_that("a tbl_ts without implicit missing values", {
   tsbl <- as_tsibble(dat_x, index = date)
   expect_identical(fill_gaps(tsbl), tsbl)
-  ref_tbl <- tibble(.from = NA, .to = NA, .n = 0L)
+  ref_tbl <- tibble(
+    .from = as.Date(character(0)), .to = as.Date(character(0)),
+    .n = integer(0)
+  )
   expect_identical(count_gaps(tsbl), ref_tbl)
 })
 
@@ -164,16 +168,27 @@ test_that("count_gaps.tbl_ts(.full = TRUE)", {
   )
 })
 
+# harvest <- tsibble(
+#   year = c(2010, 2011, 2012, 2011, 2012, 2014),
+#   fruit = rep(c("kiwi", "cherry"), each = 3),
+#   kilo = sample(1:10, size = 6),
+#   key = id(fruit), index = year
+# )
+#
+# test_that("count_gaps.tbl_ts(.full = TRUE, .common = TRUE)", {
+#   expect_equal(NROW(count_gaps(harvest, .common = TRUE)), 0)
+#   expect_equal(NROW(count_gaps(harvest, .common = TRUE, .full = TRUE)), 2)
+# })
+
 test_that("count_gaps.tbl_ts(.full = FALSE)", {
   full_tbl <- tsbl %>% count_gaps()
-  a <- tibble(group = "a", .from = NA, .to = NA, .n = 0L)
   b <- tibble(
     group = "b",
     .from = ymd("2017-01-13"),
     .to = ymd("2017-01-13"),
     .n = 1L
   )
-  expect_equal(full_tbl, dplyr::bind_rows(a, b))
+  expect_equal(full_tbl, b)
 })
 
 harvest <- tsibble(
@@ -188,8 +203,8 @@ test_that("has_gaps()", {
   expect_equal(has_gaps(harvest, .full = TRUE)$.gaps, c(TRUE, TRUE))
 })
 
-test_that("Error in gaps()", {
-  expect_error(gaps(x = 1:4, y = 1:3), "must not be greater than")
+test_that("Error in tbl_gaps()", {
+  expect_error(tbl_gaps(x = 1:4, y = 1:3), "must not be greater than")
 })
 
 test_that("seq_generator()", {
