@@ -26,11 +26,12 @@ test_that("if it's a tibble", {
   expect_equal(NROW(tsbl[1, "group"]), 1)
   expect_equal(NROW(tsbl[1:11, "group"]), 11)
   expect_is(tsbl[1, c("group", "date")], "tbl_ts")
-  expect_is(tsbl[, 1], "tbl_df")
-  expect_is(tsbl[, 2], "tbl_df")
-  expect_is(tsbl[, 3], "tbl_df")
-  expect_is(tsbl[, 2:3], "tbl_df")
-  expect_is(tsbl[, c(1, 3)], "tbl_df")
+  expect_equal(class(tsbl[, 1])[1], "tbl_df")
+  expect_equal(class(tsbl[, 2])[1], "tbl_df")
+  expect_equal(class(tsbl[, 3])[1], "tbl_df")
+  expect_equal(class(tsbl[, 2:3])[1], "tbl_df")
+  expect_equal(class(tsbl[, c(1, 3)])[1], "tbl_df")
+  expect_equal(class(tsbl[c(3, 3), ])[1], "tbl_df")
   tsbl_tmp <- tsbl %>% filter(group == "a")
   expect_is(tsbl_tmp[, "date"], "tbl_ts")
   expect_is(tsbl_tmp["date"], "tbl_ts")
@@ -47,7 +48,7 @@ test_that("if it's a tsibble", {
   expect_identical(tsbl[1:11, ], as_tibble(tsbl)[1:11, ])
   expect_equal(tsbl[], tsbl)
   expect_is(tsbl[, c(1, 2)], "tbl_ts")
-  expect_warning(tsbl[c("group", "date"), drop = TRUE], "`drop` is ignored.")
+  expect_warning(tsbl[c("group", "date"), drop = TRUE], "drop ignored")
   expect_identical(tsbl[FALSE, ], tsbl[0, ])
   expect_identical(tsbl[TRUE, ], tsbl)
   tsbl2 <- tsbl[1, ]
