@@ -46,7 +46,7 @@ test_that("gather()", {
     gather(key = key, value = value, x:z)
   expect_equal(dim(out), c(30, 3))
   expect_equal(key_vars(out), "key")
-  expect_equal(key_size(out), rep(10, 3))
+  # expect_equal(key_size(out), rep(10, 3))
   out2 <- tsbl2 %>%
     gather(key = key, value = value)
   expect_identical(out, out2)
@@ -82,7 +82,7 @@ nest_t <- tourism %>%
   nest(-Region, -State)
 
 test_that("unnest.lst_ts()", {
-  expect_error(nest_t %>% unnest(key = Region), "Key must be created")
+  expect_error(nest_t %>% unnest(key = Region), "Key can only be created")
   expect_error(nest_t %>% unnest(), "is not a valid tsibble.")
   expect_is(nest_t %>% unnest(key = id(Region, State)), "tbl_ts")
   expect_equal(nest_t %>% unnest(key = id(Region, State)), tourism)
@@ -102,7 +102,7 @@ nest2_t <- tourism %>%
   )
 
 test_that("unnest.tbl_ts()", {
-  expect_error(nest2_t %>% unnest(key = qtl), "Key must be created")
+  expect_error(nest2_t %>% unnest(key = qtl), "Key can only be created")
   expect_error(nest2_t %>% unnest(), "is not a valid tsibble.")
   expect_is(nest2_t %>% unnest(key = id(qtl)), "tbl_ts")
   expect_equal(nest2_t %>% unnest(key = id(qtl)) %>% NCOL, 6)
@@ -145,4 +145,11 @@ test_that("fill()", {
       group_by(fruit) %>% 
       fill(kilo, .direction = "down")
   )
+  expect_equal(
+    harvest_fill %>%
+      fill(kilo, .direction = "down"),
+    harvest_fill %>% 
+      as_tibble() %>% 
+      fill(kilo, .direction = "down")
+  )
 })
